/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.sun.jersey.api.client.ClientResponse;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.opengroup.osdu.register.model.Subscriber;
import org.opengroup.osdu.register.util.AzureTestUtils;
import org.opengroup.osdu.register.util.Config;
import org.opengroup.osdu.register.util.RestDescriptor;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

public class TestCreateSubscriberApi extends CreateSubscriberApiTest {

    @Before
    @Override
    public void setup() throws Exception {
        this.testUtils = new AzureTestUtils();
        super.setup();
    }

    @After
    @Override
    public void tearDown() {
        this.testUtils = null;
    }

    @Test
    @Override
    public void should_return400_when_makingHttpRequestWithoutToken() throws Exception {
        ClientResponse response = descriptor.run(getId(), "");
        assertEquals(error(response.getEntity(String.class)), 403, response.getStatus());
    }

    @Override
    protected void validate20XResponse(ClientResponse response, RestDescriptor descriptor) {
        String url = Config.Instance().PushUrl + "api/register/v1/test/challenge/1";
        String body = response.getEntity(String.class);
        Subscriber subscriber = new Gson().fromJson(body, Subscriber.class);
        String createdBy = System.getProperty("INTEGRATION_TESTER", System.getenv("INTEGRATION_TESTER"));

        assertEquals("My test listener.", subscriber.description);
        assertEquals(createdBy, subscriber.createdBy);
        assertEquals("My listener", subscriber.name);
        assertEquals("records-changed", subscriber.topic);
        assertFalse(Strings.isNullOrEmpty(subscriber.notificationId));
        assertEquals(url, subscriber.pushEndpoint);
    }

    @Test
    @Override
    public void should_onlyCreateMax1Resource_and_return409ForTheRest_when_creatingTheSameResourceInParallel() throws Exception {
        ExecutorService executor = Executors.newFixedThreadPool(8);
        List<Callable<ClientResponse>> tasks = new ArrayList<>();

        for (int i = 0; i < 8; i++) {
            Callable<ClientResponse> task = () -> {
                try {
                    return descriptor.run(getId(), testUtils.getOpsAccessToken());
                } catch (Exception ex) {
                    return null;
                }
            };
            tasks.add(task);
        }

        List<Future<ClientResponse>> responses = executor.invokeAll(tasks);
        executor.shutdown();
        executor.awaitTermination(60, TimeUnit.SECONDS);

        int non409ErrorResponseCount = 0;
        for (Future<ClientResponse> future : responses) {
            if (future.get().getStatus() != 409 && future.get().getStatus() != expectedOkResponseCode())
                non409ErrorResponseCount++;
        }

        deleteResource();
        assertEquals(error("Unexpected error response returned"), 0, non409ErrorResponseCount);
    }
}
